(function($) {
  /**
   * OpenEDU Quicktabs.
   */
  Drupal.OpenEDUQuickTabs = {
    var: {
      class: {
        // General class used to trigger OpenEDU QuickTabs.
        wrapper: 'openedu-quicktabs',

        // Generic classes.
        selected: 'selected',
        hide: 'hide',
        item: 'openedu-quicktabs-item',

        // Tab wrapper and item classes.
        tabs: 'openedu-quicktabs-tabs',
        tabs_wrapper: 'openedu-quicktabs-tabs-wrapper',
        tabs_item: 'openedu-quicktabs-tabs-item',

        // Content wrapper and item classes.
        content: 'openedu-quicktabs-content',
        content_wrapper: 'openedu-quicktabs-content-wrapper',
        content_item: 'openedu-quicktabs-content-item',
        content_item_content: 'openedu-quicktabs-content-item-content',

        // Individual groups generated by system.
        group: 'openedu-quicktabs-group',
        group_tab: 'openedu-quicktabs-group-tab',
        group_content: 'openedu-quicktabs-group-content'
      }
    },

    /**
     * Convert data attributes into inline style.
     *
     * @param {Object} obj
     *  Object being inspected.
     */
    style: function (obj) {
      var data_attrs = obj.data(),
        inline_style = '';

      if (data_attrs) {
        // Background image.
        if (data_attrs.styleBackgroundUrl) {
          inline_style += 'background-image:url(' + data_attrs.styleBackgroundUrl + ');';
        }
      }

      return (inline_style != '') ? 'style="' + inline_style + '"' : '';
    },

    /**
     * Set hide class to all elements, tabs and content.
     */
    setHideClass: function (obj) {
      var variables = this.var;

      // Remove all selected classes.
      obj.find('.' + variables.class.selected)
        .removeClass(variables.class.selected);

      // Hide all content tabs.
      obj.find('.' + variables.class.content_item)
        .addClass(variables.class.hide);
    },

    /**
     * Set selected elements, tabs and content.
     */
    setSelectedClass: function (obj, selected) {
      var variables = this.var;

      if (selected == 'first') {
        // Set tab first.
        obj.find('.' + variables.class.tabs_item)
          .first()
          .removeClass(variables.class.hide)
          .addClass(variables.class.selected);

        // Set content first.
        obj.find('.' + variables.class.content_item)
          .first()
          .removeClass(variables.class.hide)
          .addClass(variables.class.selected);
      }
      else {
        // Set selected tab.
        selected.addClass(variables.class.selected);

        // Set selected content.
        obj.find('.' + selected.data('openeduquicktabs'))
          .removeClass(variables.class.hide)
          .addClass(variables.class.selected)
          .show();
      }
    },

    /**
     * Bind tab clicking behaviour.
     */
    clickTab: function (obj) {
      var methods = this,
          variables = this.var;

      obj.find('.' + variables.class.tabs_item).on('click', function() {
        // Set vars.
        var $this = $(this);

        // Remove all selected elements and add hide to them all.
        methods.setHideClass(obj);

        // Set selected tab and content.
        methods.setSelectedClass(obj, $this);
      });
    },

    /**
     * Bind accordion clicking behaviour.
     */
    clickAccordion: function (obj) {
      var methods = this,
          variables = this.var;

      $('.' + variables.class.content + ' .' + variables.class.tabs_item).on('click', function() {
        // Set vars.
        var $this = $(this),
          parent = $this.parents('.' + variables.class.content_item),
          classes = parent.attr('class').split(' ');

        // Check if item is available.
        if (classes.length) {
          var tab_class = classes[1].replace(variables.class.content_item, variables.class.tabs_item);

          // Emulate tab clicking.
          $('.' + variables.class.tabs + ' .' + tab_class).click();
        }
      });
    },

    /**
     * Creates OpenEDU QuickTabs based on a custom structure as follow:
     *
     * [code]
     * <div class="openedu-quicktabs">
     *  <div class="openedu-quicktabs-tabs">
     *    <div class="openedu-quicktabs-wrapper">
     *      <div class="openedu-quicktabs-tabs-item openedu-quicktabs-tabs-item-X"></div>
     *      <div class="openedu-quicktabs-tabs-item openedu-quicktabs-tabs-item-Y"></div>
     *    </div>
     *  </div>
     *  <div class="openedu-quicktabs-content">
     *    <div class="openedu-quicktabs-wrapper">
     *      <div class="openedu-quicktabs-content-item">
     *        <tag class="openedu-quicktabs-tabs-item"></tag>
     *        <tag>[content]</tag>
     *      </div>
     *    </div>
     *  </div>
     * </div>
     * [/code]
     *
     * @param {string} selector
     *  Item that should be converted into QuickTabs.
     *
     * @return {String}
     *  Rendered object.
     */
    create: function (selector) {
      var methods = this,
          variables = this.var,
          group = $(selector),
          items = group.find('.' + variables.class.content_item),
          tabs = '';

      // Check if items are available so tabs can be created.
      if (items.length) {

        // Wrap content to facilitate click behaviour.
        group.wrapInner($('<div/>', {
          class: variables.class.content_wrapper,
          html: $('<div/>', {
            class: variables.class.content
          })
        }));

        // Run through items.
        items.each(function(index, value) {

          // Set item vars.
          var item = $(this),
              item_class = variables.class.tabs_item + '-' + index,
              content_class = variables.class.content_item + '-' + index,
              item_label = item.find('.' + variables.class.tabs_item),
              item_inline_style =  methods.style(item_label),
              first_item_class = (index == 0 ? ' ' + variables.class.selected : '');

          // Append item_class to item.
          item.addClass(content_class);

          // Create tab.
          tabs += '<div class="' + variables.class.tabs_item + ' ' + item_class + first_item_class + '" data-openeduquicktabs="' + content_class + '" ' + item_inline_style + '>' +
            ' <div class="internal-wrapper">' +
            '   <span class="top"></span>' +
            item_label.html() +
            '   <span class="bottom"></span>' +
            ' </div>' +
            '</div>';
        });

        class_total_items = 'openedu-quicktabs-total-tabs-' + items.length;

        // Append tabs to wrapper.
        group.prepend($('<div/>', {
          class: variables.class.tabs_wrapper,
          html: $('<div/>', {
            class: variables.class.tabs + ' ' + class_total_items,
            html: tabs
          })
        }));

        // Set hide class on all items.
        methods.setHideClass(group);

        // Set selected classe on first elements.
        methods.setSelectedClass(group, 'first');

        // Append clicking.
        methods.clickTab(group);
        //methods.clickAccordion();
      }
    }
  };

  /**
   * Creates tab display when necessary class is added.
   */
  Drupal.behaviors.OpenEDUQuickTabsX = {
    attach: function (context) {
      // Set vars.
      var selectors = Drupal.settings.OpenEDUJSQuickTabs.selectors;

      // Check if we have items to be processed.
      if (selectors.length) {
        $.each(selectors, function(k, v) {
          Drupal.OpenEDUQuickTabs.create(v);
        });
      }
    }
  };
})(jQuery);
