<?php
/**
 * @file
 * Admin options for OpenEDU Notifications Client.
 */

/**
 * Register Notifications Server form.
 */
function openedu_notifications_client_register_server_form($form, &$form_state) {
  // Set vars.
  $openedu_notifications_client_var = openedu_core_contextualized_variable_get(OPENEDU_NOTIFICATIONS_CLIENT_VAR_GENERAL, array());

  // Register server.
  $form['server'] = array(
    '#type' => 'fieldset',
    '#title' => t('Register Notifications Server'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['server']['url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#description' => t('Insert the endpoint URL that will be used to retrieve all syndicated data across installations. Ex. <i>http://endpoint.url</i>'),
    '#field_suffix' => OPENEDU_NOTIFICATIONS_ENDPOINT_SERVER,
    '#default_value' => (isset($openedu_notifications_client_var['url']) ?
      $openedu_notifications_client_var['url'] : FALSE),
    '#required' => TRUE,
  );
  $form['server']['uuid'] = array(
    '#type' => 'textfield',
    '#title' => t('Server uniq identifier'),
    '#description' => t('Request will be validated against the uniq identifier generated by the server.'),
    '#default_value' => (isset($openedu_notifications_client_var['uuid']) ?
      $openedu_notifications_client_var['uuid'] : FALSE),
    '#required' => TRUE,
  );
  $form['server']['private_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Private Token'),
    '#description' => t('Request will be validated against the token generated by the server.'),
    '#default_value' => (isset($openedu_notifications_client_var['private_token']) ?
      $openedu_notifications_client_var['private_token'] : FALSE),
    '#required' => TRUE,
  );
  $form['server']['public_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Public Token'),
    '#description' => t('Request will be validated against the token generated by the server.'),
    '#default_value' => (isset($openedu_notifications_client_var['public_token']) ?
      $openedu_notifications_client_var['public_token'] : FALSE),
    '#required' => TRUE,
  );

  // Enforce validate and submit.
  $form['#validate'][] = 'openedu_notifications_client_register_server_form_validate';
  $form['#submit'][] = 'openedu_notifications_client_register_server_form_submit';

  return system_settings_form($form);
}

/**
 * Register Notifications Server form validate.
 * In order to validate endpoint a request will be send over waiting for a
 * HTTP 200 status code.
 */
function openedu_notifications_client_register_server_form_validate(&$form, &$form_state) {
  // Set vars.
  $endpoint = $form_state['values']['url'] . OPENEDU_NOTIFICATIONS_ENDPOINT_SERVER;
  $private_token = $form_state['values']['private_token'];

  // Check if endpoint should be validated or not.
  if (TRUE === openedu_notifications_endpoint_validation()) {
    // Submit request.
    $request = openedu_authentication_http_request($endpoint, array(), 'retrieve', $private_token);

    // Check for 200 and save response from server.
    if (200 === (int) $request->code) {
      variable_set(OPENEDU_NOTIFICATIONS_CLIENT_VAR_ENDPOINTS, json_decode($request->data));
    }
    // Set feedback.
    else {
      // Set message.
      $error_message = isset($request->status_message) ? $request->status_message :
        t("Sorry, we couldn't validate the endpoint properly please, try again or review if the information provided is valid.");

      // Set error.
      form_set_error('url', $error_message);
    }
  }
}

/**
 * Register Notifications Server form submit.
 */
function openedu_notifications_client_register_server_form_submit(&$form, &$form_state) {
  // Load variable.
  $openedu_notifications_client_var = variable_get(OPENEDU_NOTIFICATIONS_CLIENT_VAR_GENERAL, array());

  // Update variable.
  variable_set(OPENEDU_NOTIFICATIONS_CLIENT_VAR_GENERAL, array(
      openedu_core_contextualized_variable_name() => array(
      'url' => $form_state['values']['url'],
      'uuid' => $form_state['values']['uuid'],
      'private_token' => $form_state['values']['private_token'],
      'public_token' => $form_state['values']['public_token']
    )
  ) + $openedu_notifications_client_var);
}

/**
 * Register entities that should be sent/shared with OpenEDU Notifications Server.
 */
function openedu_notifications_client_register_allowed_entities_form($form, &$form_state) {
  // Set vars.
  $allowed_entities = openedu_core_contextualized_variable_get(OPENEDU_NOTIFICATIONS_CLIENT_VAR_ALLOWED_ENTITIES);
  $entity_get_info = entity_get_info();

  // Set hidden field to entity_get_info() info.
  $form['entity_get_info'] = array(
    '#type' => 'hidden',
    '#value' => json_encode($entity_get_info)
  );

  // Init vertical tabs group.
  $form['allowed_entities'] = array(
    '#type' => 'vertical_tabs',
    '#weight' => 99,
  );

  foreach (entity_get_info() as $entity_type => $entity_info) {
    // Set vars.
    $fieldset = 'fieldset_' . $entity_type;
    $bundle_options = array();
    $bundle_results = isset($allowed_entities[$entity_type]) && !empty($allowed_entities[$entity_type]);

    // Bundles.
    foreach ($entity_info['bundles'] as $bundle_type => $bundle_info) {
      $bundle_options['entity:' . $entity_type . ':' . $bundle_type] = $bundle_info['label'];
    }

    // Only display options if $bundle_options is not empty.
    if (!empty($bundle_options)) {
      // Fieldset.
      $form['allowed_entities'][$fieldset] = array(
        '#type' => 'fieldset',
        '#title' => check_plain($entity_info['label']),
        '#collapsible' => TRUE,
        '#collapsed' => (TRUE === $bundle_results) ? FALSE : TRUE
      );

      // Options.
      $form['allowed_entities'][$fieldset][$entity_type] = array(
        '#type' => 'checkboxes',
        '#title' => t('Select all bundles that should trigger OpenEDU Notifications.'),
        '#options' => $bundle_options,
        '#default_value' => (TRUE === $bundle_results) ? $allowed_entities[$entity_type] : array()
      );
    }
  }

  // Enforce submit.
  $form['#submit'][] = 'openedu_notifications_client_register_allowed_entities_form_submit';

  return system_settings_form($form);
}

/**
 * Register openedu_notifications_client_register_allowed_entities_form() submission.
 */
function openedu_notifications_client_register_allowed_entities_form_submit(&$form, &$form_state) {
  // Set vars.
  $result = array();
  $values = $form_state['values'];
  $entity_get_info = json_decode($values['entity_get_info'], TRUE);
  $openedu_notifications_client_allowed_entities = variable_get(OPENEDU_NOTIFICATIONS_CLIENT_VAR_ALLOWED_ENTITIES, array());

  // Remove entity_get_info.
  unset($values['entity_get_info']);

  // Check for compatibility and empty results.
  foreach ($entity_get_info as $entity_type => $entity_info) {
    if (isset($values[$entity_type])) {
      $result[$entity_type] = array_filter($values[$entity_type]);
    }
  }

  // Update variable.
  variable_set(OPENEDU_NOTIFICATIONS_CLIENT_VAR_ALLOWED_ENTITIES, array(
      openedu_core_contextualized_variable_name() => $result
  ) + $openedu_notifications_client_allowed_entities);
}

/**
 * Register Notifications Server form.
 */
function openedu_notifications_client_entities_per_role_form($form, &$form_state) {
  // Set vars.
  $form = array();
  $_form = array();
  $allowed_entities = openedu_core_contextualized_variable_get(OPENEDU_NOTIFICATIONS_CLIENT_VAR_ALLOWED_ENTITIES, array());
  $entities_per_role = openedu_core_contextualized_variable_get(OPENEDU_NOTIFICATIONS_CLIENT_VAR_ENTITIES_PER_ROLE, array());
  $available_states = array();

  // Only display options if any is available.
  if (is_array($allowed_entities) && !empty($allowed_entities)) {
    // Load all available user roles.
    $user_roles = user_roles();

    // Is OpenEDU Workflow available and load all states.
    if (module_exists('openedu_workflow')) {
      $available_states = openedu_workflow_get_all_states();
    }

    // Based on {$allowed_entities} create all items just once to be used while
    // looping into roles.
    foreach ($allowed_entities as $entity_name => $entity_options) {
      // Only process no empty items.
      if (!empty($entity_options)) {
        // Set vars.
        $_entity_options = array();

        // Create fieldset wrapper for each entity.
        $_form[$entity_name]['fieldset'] = array(
          '#type' => 'fieldset',
          '#title' => check_plain($entity_name),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE
        );

        // Do we have different $available_states to go through?
        // Right now only node entity is been allowed to it.
        if ($entity_name === 'node' && !empty($available_states)) {
          // Set/reset vars.
          $state_options = array();

          // Increase all options appending state machine.
          foreach ($entity_options as $entity_bundle) {

            // Run through state machines if available.
            if (isset($available_states[$entity_bundle])) {
              foreach ($available_states[$entity_bundle] as $workflow_state => $state_info) {
                $state_options[$entity_bundle . ':' . $workflow_state] = $entity_bundle . ':' . $workflow_state;
              }
            }
          }

          $_entity_options = $state_options;
        }
        else {
          // Bundle options.
          $_entity_options = $entity_options;
        }

        if (!empty($_entity_options)) {
          $_form[$entity_name]['options'] = array(
            '#type' => 'checkboxes',
            '#options' => $_entity_options,
            '#default_value' => array()
          );
        }
      }
    }

    // Initiate form api options.
    // Vertical tabs group.
    $form['entities_per_role'] = array(
      '#type' => 'vertical_tabs',
      '#weight' => 99,
    );

    // Create fieldset with role names.
    foreach ($user_roles as $rid => $rname) {
      // Set vars.
      $xrname = 'x ' . $rname;

      // Set fieldset.
      $form['entities_per_role'][$rname] = array(
        '#type' => 'fieldset',
        '#title' => check_plain($rname),
        '#description' => t('Select all bundle notifications types that should be received by this role.'),
        '#collapsible' => TRUE,
      );

      // Append all options.
      foreach ($_form as $_form_entity => $_form_entity_options) {
        // Set entity fieldset.
        $form['entities_per_role'][$rname][$_form_entity] = $_form_entity_options['fieldset'];

        // Set bundle options.
        $form['entities_per_role'][$rname][$_form_entity][$xrname] = $_form_entity_options['options'];

        // Set bundle options default values.
        $form['entities_per_role'][$rname][$_form_entity][$xrname]['#default_value'] = isset($entities_per_role[$rname]) ? $entities_per_role[$rname] : array();
      }

      // Check if fieldset should be collapsed or not.
      $form['entities_per_role'][$rname]['#collapsed'] = TRUE;
    }
  }
  else {
    drupal_set_message(t('In order to set permissions is required to set the <a href="@link">allowed entities to be submitted to the server</a>.', array(
      '@link' => url(OPENEDU_NOTIFICATIONS_ADMIN_PATH . '/client/allowed-entities')
    )), 'error');
  }

  // Enforce validate and submit.
  $form['#submit'][] = 'openedu_notifications_client_entities_per_role_form_submit';

  return system_settings_form($form);
}

/**
 * Register Notifications Server form submit.
 */
function openedu_notifications_client_entities_per_role_form_submit($form, &$form_state) {
  // Set vars.
  $entities_per_role = array();
  $user_roles = user_roles();
  $input = $form_state['input'];
  $openedu_notifications_client_var = variable_get(OPENEDU_NOTIFICATIONS_CLIENT_VAR_ENTITIES_PER_ROLE, array());

  // Run through all roles in order to match submitted results.
  foreach ($user_roles as $rid => $rname) {
    $xname = 'x_' . str_replace(' ', '_', $rname);
    $xpermissions = isset($input[$xname]) ? array_filter($input[$xname]) : FALSE;

    if (!empty($xpermissions)) {
      $entities_per_role[$rname] = $xpermissions;
    }
  }

  // Update variable.
  variable_set(OPENEDU_NOTIFICATIONS_CLIENT_VAR_ENTITIES_PER_ROLE, array(
      openedu_core_contextualized_variable_name() => $entities_per_role
  ) + $openedu_notifications_client_var);

  // Redirect user.
  $form_state['redirect'] = OPENEDU_NOTIFICATIONS_ADMIN_PATH . '/client/entities-per-role';
}
