<?php
/**
 * @file
 * Admin options for OpenEDU Notifications Client.
 */

/**
 * Register Notifications Client endpoint form.
 */
function openedu_notifications_server_register_endpoints_form($form, &$form_state) {
  // Set vars.
  $path = drupal_get_path('module', 'openedu_notifications_server');
  $openedu_notifications_server_var = variable_get(OPENEDU_NOTIFICATIONS_SERVER_VAR, array());
  $endpoints_table = openedu_notifications_server_endpoints_table();

  // Register endpoints.
  $form['endpoint'] = array(
    '#type' => 'fieldset',
    '#title' => t('Register Notifications Client Endpoint'),
    '#description' => t('A <i>token</i> will be generated by the system once the endpoint is validated.'),
    '#collapsible' => TRUE,
    '#collapsed' => (!empty($endpoints_table) ? TRUE : FALSE),
    '#prefix' => '<div class="endpoint-table">' . $endpoints_table . '</div>',
    '#suffix' => '',
  );
  $form['endpoint']['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#description' => t('Title is only used for reference.'),
    '#default_value' => '',
    '#required' => TRUE,
  );
  $form['endpoint']['url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#description' => t('URL used during requests.'),
    '#field_suffix' => OPENEDU_NOTIFICATIONS_ENDPOINT_CLIENT,
    '#default_value' => '',
    '#required' => TRUE,
  );
  $form['endpoint']['eid'] = array(
    '#type' => 'hidden',
    '#default_value' => ''
  );

  // Attach necessary CSS and JS.
  $form['#attached']['css'][] = $path . '/openedu_notifications_server.admin.css';
  $form['#attached']['js'][] = $path . '/openedu_notifications_server.admin.js';

  // Enforce validate and submit.
  $form['#validate'][] = 'openedu_notifications_server_register_endpoints_form_validate';
  $form['#submit'][] = 'openedu_notifications_server_register_endpoints_form_submit';

  return system_settings_form($form);
}

/**
 * Return table to be displayed if an endpoint is already registered.
 *
 * @return bool|string
 */
function openedu_notifications_server_endpoints_table() {
  // Check if endpoints are available.
  if (($registered_endpoints = openedu_notifications_server_endpoint_retrieve()) == FALSE) {
    return FALSE;
  }

  global $base_url;

  // Set vars.
  $rows = array();
  $admin_url = $base_url . '/' . OPENEDU_NOTIFICATIONS_ADMIN_PATH . '/server/endpoint';

  // And convert all of them into array, which is required by theme().
  foreach ($registered_endpoints as $k => $endpoint) {
    // Set vars.
    $data_attr = '';

    // Create data attributes to be appended.
    foreach ($endpoint as $field => &$value) {
      // Format date.
      if ($field == 'created') {
        $value = format_date($value, 'short');
      }

      // Set data attribute to be used during edit.
      $data_attr .= 'data-' . $field . '="' . $value . '"';
    }

    // Convert object into an array.
    $rows[$k] = (array) $endpoint;

    // Change status to display text.
    $rows[$k]['status'] = ($endpoint->status == 1) ? t('Enabled') : t('Disabled');

    // Change tokens column to display both values.
    $rows[$k]['token_private'] = <<<TOKENS
<b>UUID:</b> {$rows[$k]['uuid']}<br />
<b>Private:</b> {$rows[$k]['token_private']}<br />
<b>Public:</b> {$rows[$k]['token_public']}
TOKENS;

    // Remove {uuid}, {token_private} and {data} from display.
    unset($rows[$k]['uuid'], $rows[$k]['token_public'], $rows[$k]['data']);

    // Add butons to allow user to approve, edit and delete endpoints.
    switch ($endpoint->status) {
      case -1 :
        $approval = '<li><a href="' . $admin_url . '/' . $endpoint->eid . '/enable" title="Approve item" class="approve">Approve</a></li>';
        break;
      case 1 :
        $approval = '<li><a href="' . $admin_url . '/' . $endpoint->eid . '/disable" title="Disable item" class="disable">Disable</a></li>';
        break;
      case 0 :
        $approval = '<li><a href="' . $admin_url . '/' . $endpoint->eid . '/enable" title="Enable item" class="approve">Enable</a></li>';
        break;
    }
    $rows[$k]['actions'] = <<<ACTIONS
      <ul class="actions" $data_attr>
        $approval
        <li><a href="{$admin_url}/{$endpoint->eid}/edit" title="Edit item" class="edit">Edit</a></li>
        <li><a href="{$admin_url}/{$endpoint->eid}/remove" title="Remove item" class="remove">Remove</a></li>
      </ul>
ACTIONS;
  }

  // Append new title.
  $label = '<h1>' . t('Registered endpoints') . '</h1>';

  // Create table.
  $table = theme('table', array(
    'header' => array(
      'ID',
      'Status',
      'Name',
      'URL',
      'Tokens',
      'Creation',
      'Actions'
    ),
    'rows' => $rows
  ));

  return $label . $table;
}

/**
 * Register Notifications Client endpoint form validate.
 */
function openedu_notifications_server_register_endpoints_form_validate(&$form, &$form_state) {
  // Set vars.
  $values = $form_state['values'];
  $check_url = TRUE;

  // Check if {eid} is present, which means user is trying to update content.
  if (isset($values['eid']) && is_numeric($values['eid'])) {
    // Load endpoint based on its eid.
    $endpoint = openedu_notifications_server_endpoint_load(array('eid' => $values['eid']));

    // Check if endpoint is present.
    if (isset($endpoint) && !empty($endpoint)) {
      // Check if user is editing the URL.
      // Being equals means user is trying to edit any other information that not
      // url which allow us to bypass URL validation, since it was validated before.
      if ($endpoint->url == trim($values['url'])) {
        $check_url = FALSE;
      }
    }
    // If false something went wrong, notify user and escape process.
    else {
      drupal_set_message(t('Your endpoint could not be found. Please try again.'), 'error');
      drupal_goto(OPENEDU_NOTIFICATIONS_ADMIN_PATH . '/server/endpoint/register');
    }
  }

  // Check if URL already exists in DB.
  if (TRUE === $check_url) {
    if (openedu_notifications_server_endpoint_load(array('url' => $values['url'])) == TRUE) {
      form_set_error('url', t('Sorry, this is a duplicate URL endpoint. Please try another endpoint value.'));
    }
  }
}

/**
 * Register Notifications Client endpoint form submit.
 */
function openedu_notifications_server_register_endpoints_form_submit(&$form, &$form_state) {
  // Set vars.
  $values = $form_state['values'];
  $key = NULL;

  // Check is an id is being sent in order to update an item.
  if (isset($values['eid']) && is_numeric($values['eid'])) {
    $record = array(
      'eid' => $values['eid'],
      'title' => $values['title'],
      'url' => $values['url'],
    );

    // Set key to be validated against.
    $key = 'eid';
  }
  // Or insert a new one, endpoint is enabled by default.
  else {
    $record = array(
      'status' => 1,
      'title' => $values['title'],
      'url' => $values['url']
    );
  }

  return openedu_notifications_server_register_endpoint($record, $key);
}

/**
 * OpenEDU Notifications Client: Remove endpoint.
 * Confirms if user really wanna remove and endpoint.
 */
function openedu_notifications_server_endpoint_remove($form, &$form_state, $endpoint_id) {
  // Set vars.
  $redirect_url = OPENEDU_NOTIFICATIONS_ADMIN_PATH . '/server/endpoint/register';
  $endpoint = openedu_notifications_server_endpoint_retrieve(array('eid' => $endpoint_id));

  // Check if endpoint is valid.
  if (FALSE == $endpoint || empty($endpoint)) {
    drupal_set_message(t('Sorry, ID submitted is not valid.'), 'error');
    drupal_goto($redirect_url);
  }

  // Create and set a field to carry the endpoint id will be removed.
  $form = array(
    'endpoint' => array(
      '#type' => 'value',
      '#value' => $endpoint_id,
    )
  );

  // Generates a form array for a confirmation form.
  return confirm_form(
    $form,
    t('Are you sure you want to delete <i>:title</i> endpoint?', array(
      ':title' => $endpoint[0]->title
    )),
    $redirect_url,
    t('This action cannot be undone.'),
    t('Remove endpoint'),
    t('Cancel')
  );
}

/**
 * OpenEDU Notifications Client: Remove endpoint.
 * Execute removal.
 */
function openedu_notifications_server_endpoint_remove_submit(&$form, &$form_state) {
  // Remove item from DB and set feedback message.
  if (TRUE == openedu_notifications_server_endpoint_delete($form_state['values']['endpoint'])) {
    drupal_set_message(t('Endpoint removed successfully.'));
  }
  else {
    drupal_set_message(t('Endpoint could not be removed.'), 'error');
  }

  $form_state['redirect'] = OPENEDU_NOTIFICATIONS_ADMIN_PATH . '/server/endpoint/register';
}
