<?php
/**
 * @file
 * OpenEDU Notifications Server endpoint configuration.
 */

/**
 * Access callback for the server endpoint and allow users to retrieve all data
 * since passed token validation.
 *
 * @param string $op
 *  The operation that's going to be performed.
 *
 * @return bool
 *  Whether access is given or not.
 */
function openedu_notifications_server_resource_access($op) {
  switch ($op) {
    // Index, Update, Delete calls.
    case 'index':
    case 'update':
    case 'delete':
    case 'create':
      $access = TRUE;
      break;

    default:
      $access = FALSE;
  }

  return $access;
}

/**
 * Access callback for the notifications endpoint and allow users to retrieve
 * all data since passed token validation.
 *
 * @param string $op
 *  The operation that's going to be performed.
 *
 * @return bool
 *  Whether access is given or not.
 */
function openedu_notifications_notifications_resource_access($op) {
  switch ($op) {
    // Index, Update, Delete calls.
    case 'index':
    case 'update':
    case 'delete':
    case 'create':
      $access = TRUE;
      break;

    default:
      $access = FALSE;
  }

  return $access;
}

/**
 * Callback for listing endpoints stored on server.
 *
 * @return string
 *  JSON string.
 */
function openedu_notifications_server_resource_index($params) {
  // Retrieve all valid endpoints.
  $endpoints = openedu_notifications_server_endpoint_retrieve(array(
    'status' => 1
  ), array(
    'title',
    'url',
    'token_public'
  ));

  // Replace {token_public} to become {token}.
  if (!empty($endpoints)) {
    foreach ($endpoints as $k => $v) {
      // Rename {token_public} to {token}
      if (isset($v->token_public)) {
        $v->token = $v->token_public;
        unset($v->token_public);
      }
    }
  }

  // Return endpoints if available.
  return $endpoints;
}

/**
 * Callback for updating endpoints.
 *
 * @param array $data
 *
 * @return object
 */
function openedu_notifications_server_resource_update($token, $data) {
  // Load requester info from OpenEDU Server Validation.
  global $_openedu_authentication_endpoint_requester;

  // Check if $token matches previous validation.
  if ($_openedu_authentication_endpoint_requester->token_private != $token) {
    return services_error('Invalid token', 403);
  }

  // Check if {data[data]} is present and serialize information prior to saving.
  if (isset($data['data']) && !empty($data['data'])) {
    $data['data'] = serialize($data['data']);
  }

  // Try updating endpoint.
  $update = openedu_notifications_server_endpoint_write_record(array(
      'token_private' => $token
    ) + $data, 'token_private');

  // Check if update was executed.
  if (SAVED_UPDATED === $update) {
    $return = array(
      'token' => $token,
    );
  }
  // Something went wrong.
  else {
    $return = array();
  }

  return $return;
}

/**
 * Callback for creating an endpoint.
 *
 * @param array $data
 *   An array of parameters used to create an endpoint pragmatically.
 *   Available keys: title (string), url (string).
 *
 * @return object
 */
function openedu_notifications_server_resource_create($data) {
  // Check if {title} and {url} are present.
  if (empty($data['title']) || empty($data['url'])) {
    return services_error('Missing attribute, ensure all mandatory fields are being sent', 406);
  }

  // Validate URL checking its syntax.
  if (!valid_url($data['url'], TRUE)) {
    return services_error('Invalid URL', 406);
  }

  // Check if URL is already registered within the system.
  $endpoint = openedu_notifications_server_endpoint_load(array('url' => $data['url']));

  // Check if endpoint is present.
  if (isset($endpoint) && !empty($endpoint)) {
    return services_error('Duplicate endpoint, ensure the URL is unique.', 406);
  }

  // Try creating endpoint.
  $register_endpoint = openedu_notifications_server_register_endpoint(array(
    'status' => -1,
    'title' => $data['title'],
    'url' => $data['url'],
  ));

  // Check if submission went well.
  if (isset($register_endpoint['status']) && 200 === $register_endpoint['status']) {
    return $register_endpoint;
  }

  // Defaults to failure.
  return services_error($register_endpoint['message'], $register_endpoint['status']);
}

/**
 * Callback for deleting an endpoint.
 *
 * @param string $token
 *   Private token used to identify endpoint.
 *
 * @return object
 */
function openedu_notifications_server_resource_delete($token) {
  // Try updating endpoint simply changing its status to -1.
  $delete = openedu_notifications_server_resource_update($token, array(
    'status' => 0
  ));

  if (isset($delete['token']) && $delete['token'] === $token) {
    return array(
      'status' => 200,
    );
  }

  // Defaults to failure.
  return services_error('Failed deleting endpoint', 406);
}
